/*
Sprite_tms hack to communicate with a Rigol DS1000-series scope using Linux. 
This code is licensed under the GPL V3.

Warning: This code can in theory fubar the communications with the scope to a
point where the Linux USB-stack seems to get confused. Do a 
rmmod uhci_hcd; modprobe uhci_hcd
(or alternately: use ohci_hcd) if that happens and you should be fine.
 */

#include <usb.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>

#include <readline/readline.h>
#include <readline/history.h>

#include "usbtmc.h"
#include "commands.h"

#define MIN(a,b) (((a)<(b))?(a):(b))

inline char printable (char ch)
{
	if (ch < ' ') return '.';
	if (ch > '~') return '.';
	return ch;
}

//Debugging: Print a buffers contents in hex
void printb (unsigned char *pkt, int len)
{
	int i, j;

	for (i=0;i<len;i+= 16) {
		printf ("%04x:  ", i);
		for (j=0;j < MIN(len-i, 16);j++) {
			printf (" %02x", pkt[i+j]);
			if (j == 7) printf ("  ");
		}
		if (j < 7) printf ("  ");
		for (;j<17;j++)
			printf ("   ");

		for (j=0;j < MIN(len-i, 16);j++) {
			printf ("%c", printable (pkt[i+j]));
		}
		printf ("\n");
	}
}

#define HAVE_READLINE
#ifndef HAVE_READLINE

char *readline (prompt)
{
	char *buf;

	printf (prompt); 
	fflush (stdout); 
	buf = malloc (1024); 

	if (fgets (buf, 1023, stdin) == NULL) {
		free (buf); 
		return NULL; 
	}
	l = strlen (buf); 
	if (buf[l-1] == '\n') {
		buf[l] = 0;
	}
	return buf; 
}

void add_history (char *buf)
{
}
#endif

void child_reaper(int sig)
{
	pid_t child;

	do {
		child = waitpid(-1, NULL, WNOHANG);
	} while(child > 0);
	
}

int main(int argc, char **argv) 
{
	struct usb_dev_handle *sc;
	char *scpi;
	unsigned char *buff;
	int l;
	struct sigaction act;

	//Initialize scope
	sc = usbtmc_initscope();
	buff = malloc (1024*1024); 
	if (buff == NULL) {
		perror("malloc");
		exit(EXIT_FAILURE);
	}

	bzero(&act, sizeof(act));
	act.sa_handler = child_reaper;
	act.sa_flags = SA_NOCLDSTOP|SA_RESTART;
	if (sigaction(SIGCHLD, &act, NULL) == -1) {
		perror("sigaction");
		exit(EXIT_FAILURE);
	}

	while (1) {
		scpi = readline ("> ");

		if (!scpi) break;
		if (strlen (scpi) == 0) {
			free (scpi); 
			continue;
		}

		add_history (scpi); 

		if (strncmp (scpi, "quit", 4) == 0) break;
		if (strncmp (scpi, "plot", 4) == 0) {
			do_plot (sc);
			continue;
		}
		if (strncmp (scpi, "databuf", 7) == 0) {
			do_get_buf(sc);
			continue;
		}
		if (strncmp (scpi, "screen", 6) == 0) {
			do_get_screen(sc);
			continue;
		}
		if (strncmp (scpi, "display", 7) == 0) {
			do_display_screen(sc);
			continue;
		}

		l = strlen (scpi); 
		//printf ("got buf(%d): ", l); 
		//printb (scpi, l+2); 
		if (strchr (scpi, '?')) {
			//printf ("Expecting reply\n");
			l = usbtmc_sendscpi(sc, scpi, buff, 1024*1024);
			//printf ("Got replylen = %d.\n", l); 
			buff[l] = 0; //zero-terminate
			printb (buff, l);
		} else {
			//printf ("No reply expected\n");
			l=usbtmc_sendscpi(sc,scpi,NULL,0);
		}
		free (scpi);
	}
	//Disable keylock, so the user doesn't have to press the 'force'-button
	l=usbtmc_sendscpi(sc, ":KEY:LOCK DISABLE",NULL,0); 

	usbtmc_close(sc);
	return 0;
}
